package de.workshopsoftwarearchitektur.mmdemo;

import java.io.ByteArrayOutputStream;
import java.lang.reflect.Field;
import java.util.Collection;
import java.util.List;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.Duration;

import org.junit.Test;
import org.modelmapper.Converter;
import org.modelmapper.ModelMapper;
import org.modelmapper.PropertyMap;
import org.modelmapper.spi.MappingContext;

import de.workshopsoftwarearchitektur.mmdemo.basemodel.CD;
import de.workshopsoftwarearchitektur.mmdemo.basemodel.TestData;
import de.workshopsoftwarearchitektur.mmdemo.basemodel.Track;
import de.workshopsoftwarearchitektur.mmdemo.xmlmodel.XmlCd;
import de.workshopsoftwarearchitektur.mmdemo.xmlmodel.XmlCdCollection;
import de.workshopsoftwarearchitektur.mmdemo.xmlmodel.XmlTrack;

public class XmlTest {

	private XmlCdCollection makeXml(Collection<CD> cds) {
		XmlCdCollection result = new XmlCdCollection();

		ModelMapper mapper = new ModelMapper();
		mapper.addMappings(new PropertyMap<CD, XmlCd>() {
			@Override
			protected void configure() {
				map(source.getTracks()).setXmlTrack(null);
			}
		});
		mapper.addMappings(new PropertyMap<Track, XmlTrack>() {
			@Override
			protected void configure() {
				using(new Converter<Integer, Duration>() {
					public Duration convert(MappingContext<Integer, Duration> context) {
						try {
							return DatatypeFactory.newInstance().newDuration(context.getSource() * 1000);
						} catch (DatatypeConfigurationException e) {
							throw new RuntimeException(e);
						} finally {
							resetShading(context, "length");
						}
					}

					private void resetShading(MappingContext<Integer, Duration> context, String path) {
						try {
							Field shadedPaths = context.getClass().getDeclaredField("shadedPaths");
							shadedPaths.setAccessible(true);
							((List<String>) shadedPaths.get(context)).remove(path);
						} catch (Exception e) {
							throw new RuntimeException(e);
						}
					}
				}).map(source.getLength()).setLength(null);
			}
		});

		for (CD cd : cds)
			result.getXmlCd().add(mapper.map(cd, XmlCd.class));

		return result;
	}

	@Test
	public void dataInXml() throws Exception {
		XmlCdCollection cds = makeXml(TestData.getCds());

		System.out.println(marshal(cds));
	}

	private String marshal(XmlCdCollection xmlCdCollection) throws JAXBException {
		Marshaller marshaller = JAXBContext.newInstance("de.workshopsoftwarearchitektur.mmdemo.xmlmodel")
				.createMarshaller();

		marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
		marshaller.setProperty(Marshaller.JAXB_ENCODING, "UTF-8");

		ByteArrayOutputStream out = new ByteArrayOutputStream();
		marshaller.marshal(xmlCdCollection, out);

		return out.toString();
	}

}
